package com.yunzhi.niucai.alipay;

import com.alipay.api.AlipayApiException;
import com.alipay.api.AlipayClient;
import com.alipay.api.CertAlipayRequest;
import com.alipay.api.DefaultAlipayClient;
import com.alipay.api.request.AlipaySystemOauthTokenRequest;
import com.alipay.api.response.AlipaySystemOauthTokenResponse;
import com.yunzhi.niucai.common.UrlQueryUtils;
import com.yunzhi.niucai.entity.TaAlipay;
import com.yunzhi.niucai.service.ITaAlipayService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

@Slf4j
@Component
public class AlipayFactory implements InitializingBean {
    private AlipayClient alipayClient;

    @Resource(name = "taShopAppServiceImpl")
    private ITokenResolve tokenResolve;

    @Autowired
    private Config config;

    @Autowired
    ITaAlipayService iTaAlipayService;

    public AliPayService getService(Integer shopId) {
        return new AliPayService(shopId, alipayClient, tokenResolve, config);
    }

    private AlipayClient getAlipayClient(Config config) throws Exception {
        try {
            String privateKey = Utils.readFile(config.getPrivateKey());

            if ("".equals(privateKey.trim())) {
                String error = "读取支付宝私有密钥失败: 密钥为空";
                log.error(error);
                throw new Exception(error);
            }

            CertAlipayRequest certParams = new CertAlipayRequest();
            certParams.setServerUrl(config.getApiUrl());
            certParams.setAppId(config.getAppid());
            certParams.setPrivateKey(privateKey);
            certParams.setCharset(config.getCharset());
            certParams.setFormat(config.getFormat());
            certParams.setSignType(config.getSignType());
            certParams.setCertPath(config.getCertPath());
            certParams.setAlipayPublicCertPath(config.getPublicCertPath());
            certParams.setRootCertPath(config.getRootCertPath());

            return new DefaultAlipayClient(certParams);
        } catch (Exception e) {
            e.printStackTrace();
            log.error("初始化支付宝客户端失败: {}", e.getMessage());
            throw e;
        }
    }

    public String getAppID() {
        return config.getAppid();
    }

    /**
     * 获取授权用户ID
     * @param authCode
     * @return
     * @throws Exception
     */
    public String getUserId(String authCode) throws Exception {
        AlipaySystemOauthTokenRequest oauthTokenRequest = new AlipaySystemOauthTokenRequest();
        oauthTokenRequest.setCode(authCode);
        oauthTokenRequest.setGrantType("authorization_code");
        try {
            AlipaySystemOauthTokenResponse oauthTokenResponse = alipayClient.execute(oauthTokenRequest);
            return oauthTokenResponse.getUserId();
        } catch (AlipayApiException e) {
            log.error("授权支付宝用户信息失败: {}", e.getErrMsg());
            e.printStackTrace();
            throw e;
        }
    }

    public String getAuthURL(Map<String, String> params) {
        String authURL = config.getAuthUrl();
        String authCallback = config.getAuthCallback();

        // 组装回调地址
        String app = "alipay";
        String event = "auth";
        params.putIfAbsent("app", app);
        params.putIfAbsent("event", event);
        String redirectUri = authCallback + "?" + UrlQueryUtils.compose(params);

        // 组装授权URL
        Map<String, String> authParams = new HashMap<String, String>(){{
            put("app_id", config.getAppid());
            put("application_type", "WEBAPP");
            put("redirect_uri", redirectUri);
        }};

        return authURL + "?" + UrlQueryUtils.compose(authParams);
    }

    @Override
    public void afterPropertiesSet() throws Exception {
        // 写死, 取第一个
//        TaAlipay taAlipay = iTaAlipayService.getById(1);
//        this.config.setAppid(taAlipay.getAppid())
//                .setPrivateKey(taAlipay.getMerchantPrivateKey())
//                .setPublicKey(taAlipay.getAlipayPublicKey());

        //
        try {
            this.alipayClient = getAlipayClient(this.config);
        } catch (Exception e) {
            // 有错直接让程序挂掉
            System.exit(1);
        }
    }
}
